"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExpectResult = exports.ExpectToBeDeep = exports.ExpectToBe = exports.ExpectTo = exports.Expect = exports.expectHttpsError = exports.expectResult = exports.expect = void 0;
const chai_1 = require("chai");
function expect(value) {
    return new Expect(value);
}
exports.expect = expect;
function expectResult(result) {
    return new ExpectResult(result);
}
exports.expectResult = expectResult;
function expectHttpsError(execute, code) {
    try {
        execute();
        chai_1.expect.fail('Expected to throw an error.');
    }
    catch (error) {
        (0, chai_1.expect)(error).to.have.ownProperty('httpErrorCode');
        (0, chai_1.expect)(error).to.have.ownProperty('code');
        (0, chai_1.assert)(typeof error === 'object' && error !== null && 'code' in error);
        (0, chai_1.expect)(error.code).to.be.equal(code);
    }
}
exports.expectHttpsError = expectHttpsError;
class Expect {
    _value;
    constructor(_value) {
        this._value = _value;
    }
    value(key) {
        return new Expect(this._value[key]);
    }
    get to() {
        return new ExpectTo(this._value);
    }
}
exports.Expect = Expect;
class ExpectTo {
    _value;
    constructor(_value) {
        this._value = _value;
    }
    get be() {
        return new ExpectToBe(this._value);
    }
}
exports.ExpectTo = ExpectTo;
class ExpectToBe {
    _value;
    constructor(_value) {
        this._value = _value;
    }
    get deep() {
        return new ExpectToBeDeep(this._value);
    }
    equal(value, message) {
        return (0, chai_1.expect)(this._value).to.be.equal(value, message);
    }
    unsorted(value, message) {
        (0, chai_1.assert)(Array.isArray(this._value));
        (0, chai_1.expect)(this._value.length).to.be.equal(value.length);
        for (const element of value) {
            const index = this._value.findIndex(e => JSON.stringify(e) === JSON.stringify(element));
            if (index === -1)
                chai_1.assert.fail(message ?? `Couldn't find element: ${element}`);
            this._value.splice(index, 1);
        }
    }
}
exports.ExpectToBe = ExpectToBe;
class ExpectToBeDeep {
    _value;
    constructor(_value) {
        this._value = _value;
    }
    equal(value, message) {
        return (0, chai_1.expect)(this._value).to.be.deep.equal(value, message);
    }
    unsorted(value, message) {
        (0, chai_1.assert)(Array.isArray(this._value));
        (0, chai_1.expect)(this._value.length).to.be.equal(value.length);
        for (const element of value) {
            const index = this._value.findIndex(e => JSON.stringify(e) === JSON.stringify(element));
            if (index === -1)
                chai_1.assert.fail(message ?? `Couldn't find element: ${element}`);
            this._value.splice(index, 1);
        }
    }
}
exports.ExpectToBeDeep = ExpectToBeDeep;
class ExpectResult {
    result;
    constructor(result) {
        this.result = result;
    }
    get success() {
        if (this.result.state === 'failure') {
            console.error(this.result.error.code, this.result.error.message);
            console.error(this.result.error);
        }
        expect(this.result.state).to.be.equal('success');
        (0, chai_1.assert)(this.result.state === 'success');
        return typeof this.result.value === 'object' ? new ExpectToBeDeep(this.result.value) : new ExpectToBe(this.result.value);
    }
    get failure() {
        expect(this.result.state).to.be.equal('failure');
        (0, chai_1.assert)(this.result.state === 'failure');
        return new ExpectToBeDeep({
            code: this.result.error.code,
            message: this.result.error.message
        });
    }
}
exports.ExpectResult = ExpectResult;
